package com.llw.libjava.commons.util;

import java.util.Collection;
import java.util.Map;

public class AssertUtil {

	private AssertUtil() {
	}

	public static void assertNotNull(Object object) {
		if (object == null) {
			throw new IllegalArgumentException("null object");
		}
	}

	public static void assertNotEmpty(String str) {
		if (StringUtil.isEmpty(str)) {
			throw new IllegalArgumentException("empty string");
		}
	}

	public static void assertNotEmptyAfterTrim(String str) {
		if (StringUtil.isEmptyAfterTrim(str)) {
			throw new IllegalArgumentException("empty string");
		}
	}

	public static <T> void assertNotEmpty(Collection<T> collection) {
		if (collection == null || collection.isEmpty()) {
			throw new IllegalArgumentException("empty collection");
		}
	}

	public static <K, V> void assertNotEmpty(Map<K, V> map) {
		if (map == null || map.isEmpty()) {
			throw new IllegalArgumentException("empty map");
		}
	}

	public static void assertEquivalentString(String a, String b) {
		if ((a == null && b != null) || (a != null && !a.equals(b))) {
			throw new IllegalArgumentException("unequal string");
		}
	}

	public static void assertEquivalentNumericalValue(byte a, byte b) {
		if (a != b) {
			throwUnequalNumericalValue();
		}
	}

	public static void assertEquivalentNumericalValue(int a, int b) {
		if (a != b) {
			throwUnequalNumericalValue();
		}
	}

	public static void assertEquivalentNumericalValue(long a, long b) {
		if (a != b) {
			throwUnequalNumericalValue();
		}
	}

	private static void throwUnequalNumericalValue() {
		throw new IllegalArgumentException("unequal numerical value");
	}
}
