package com.llw.libjava.commons.util;

import java.math.BigInteger;
import java.text.DecimalFormat;

public class ByteUtil {

	private ByteUtil() {
	}

	public static String transformByteArray2HexString(byte[] byteArray) {
		if (byteArray == null || byteArray.length == 0) {
			// ExceptionUtil.throwNullArgumentException();
			return null;
		}
		int hexStringLength = byteArray.length << 1;
		StringBuilder hexStringBuilder = new StringBuilder(hexStringLength);
		for (byte oneByte : byteArray) {
			int integer = (int) oneByte;
			if (integer < 0) {
				integer += 256;
			}
			String byteChar = Integer.toHexString(integer).toUpperCase();
			if (byteChar.length() == 1) {
				byteChar = "0" + byteChar;
			}
			hexStringBuilder.append(byteChar);
		}
		String hexString = hexStringBuilder.toString();
		return hexString;
	}

	public static String transformByteArray2DecimalString(byte[] byteArray) {
		String digitStringRadix16 = transformByteArray2HexString(byteArray);
		BigInteger digitBigInteger = new BigInteger(digitStringRadix16, 16);
		String digitStringRadix10 = digitBigInteger.toString(10);
		return digitStringRadix10;
	}

	public static int transformByteArray2Integer(byte[] byteArray) {
		if (byteArray == null || byteArray.length == 0 || byteArray.length > 4) {
			throw new IllegalArgumentException();
		}
		String digitStringRadix10 = transformByteArray2DecimalString(byteArray);
		return Integer.parseInt(digitStringRadix10);
	}

	public static long transformByteArray2Long(byte[] byteArray) {
		if (byteArray == null || byteArray.length == 0 || byteArray.length > 8) {
			throw new IllegalArgumentException();
		}
		String digitStringRadix10 = transformByteArray2DecimalString(byteArray);
		return Long.parseLong(digitStringRadix10);
	}

	/**
	 * @Description transformInteger2ByteArray(int digit)
	 */
	@Deprecated
	public static byte[] transformDigit2ByteArray(int digit) {
		return transformDigitString2ByteArray(10, String.valueOf(digit));
	}

	/**
	 * @Description transformLong2ByteArray(long digit)
	 */
	@Deprecated
	public static byte[] transformDigit2ByteArray(long digit) {
		return transformDigitString2ByteArray(10, String.valueOf(digit));
	}

	/**
	 * @Description transformDigitString2ByteArray(String digit, int radix)
	 */
	@Deprecated
	public static byte[] transformDigit2ByteArray(String digit, int radix) {
		return transformDigitString2ByteArray(10, digit);
	}

	public static byte[] transformInteger2ByteArray(int digit) {
		return transformDigitString2ByteArray(10, String.valueOf(digit));
	}

	public static byte[] transformInteger2ByteArray(int digit, int byteLength) {
		return transformDigitString2ByteArray(10, String.valueOf(digit), byteLength);
	}

	public static byte[] transformLong2ByteArray(long digit) {
		return transformDigitString2ByteArray(10, String.valueOf(digit));
	}

	public static byte[] transformLong2ByteArray(long digit, int byteLength) {
		return transformDigitString2ByteArray(10, String.valueOf(digit), byteLength);
	}

	public static byte[] transformDigitString2ByteArray(int radix, String digit) {
		BigInteger digitBigInteger = new BigInteger(digit, radix);
		return digitBigInteger.toByteArray();
	}

	public static byte[] transformDigitString2ByteArray(int radix, String digit, int byteLength) {
		byte[] byteArray = transformDigitString2ByteArray(radix, digit);
		int offset = byteLength - byteArray.length;
		if (offset < 0) {
			throw new IllegalArgumentException("byte length is less than byte array size of digit value");
		} else if (offset == 0) {
			return byteArray;
		} else {
			byte[] byteArrayExtend = new byte[byteLength];
			System.arraycopy(byteArray, 0, byteArrayExtend, offset, byteArray.length);
			return byteArrayExtend;
		}
	}

	public static byte[] transformHexString2ByteArray(String hexString) {
		if (hexString == null || hexString.equals("")) {
			// ExceptionUtil.throwNullArgumentException();
			return null;
		}
		int hexStringLength = hexString.length();
		if (hexStringLength % 2 != 0) {
			throw new IllegalArgumentException("The length of hexadecimal string must be an even number!");
		}
		int byteArrayLength = hexStringLength >> 1;
		byte[] byteArray = new byte[byteArrayLength];
		char[] hexCharArray = hexString.toCharArray();
		for (int i = 0; i < hexStringLength; i += 2) {
			int integer = Integer.parseInt(String.valueOf(hexCharArray, i, 2), 16);
			byteArray[i >> 1] = (byte) integer;
		}
		return byteArray;
	}

	/**
	 * @description instead by NetUtil.ipValueOf
	 * @param ipArray
	 * @return
	 */
	@Deprecated
	public static byte[] ipValueOf(int[] ipArray) {
		return NetUtil.ipValueOf(ipArray);
	}

	public static int byte2UnsignedInt(byte b) {
		int i = b;
		return i < 0 ? (i + 256) : i;
	}

	public static int[] byte2IntArray(byte b, int[] parts) {
		if (parts == null || parts.length > 8) {
			throw new IllegalArgumentException("int[] array is empty, or array size is not in [1, 8]");
		}
		int partSum = 0;
		for (int i = 0; i < parts.length; i++) {
			partSum += parts[i];
		}
		if (partSum != 8) {
			throw new IllegalArgumentException("the sum of part value must be 8");
		}
		int originalInteger = byte2UnsignedInt(b);
		String originalString = new DecimalFormat("00000000").format(Integer.parseInt(Integer.toBinaryString(originalInteger)));
		int[] values = new int[parts.length];
		int offset = 0;
		for (int i = 0; i < values.length; i++) {
			int endIndex = offset + parts[i];
			String temp = originalString.substring(offset, endIndex);
			offset = endIndex;
			values[i] = Integer.parseInt(temp, 2);
		}
		return values;
	}
}
