package com.llw.libjava.commons.util.argument;

import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;

import com.llw.libjava.commons.constant.SymbolConstants;

import javolution.util.FastMap;

public class ArgumentPairPattern implements Iterable<ArgumentPair> {
	private Map<String, ArgumentPair> argumentPairMap = null;

	private ArgumentPairPattern() {
	}

	private void initialize(String[] arguments) {
		if (arguments == null || arguments.length % 2 != 0) {
			throw new IllegalArgumentException("The argument number is not an even number.");
		}
		argumentPairMap = new FastMap<String, ArgumentPair>();
		for (int i = 0; i < arguments.length; i += 2) {
			if (arguments[i] == null || !arguments[i].startsWith(SymbolConstants.LINE_TRANSVERSE_MIDDLE)) {
				throw new IllegalArgumentException("There is argument name which does not start with symbol '-'.");
			}
			String argumentName = arguments[i];
			String argumentValue = arguments[i + 1];
			if (argumentValue == null) {
				argumentValue = "";
			}
			ArgumentPair argumentPair = new ArgumentPair();
			argumentPair.setSequenceNo(i >> 1);
			argumentPair.setName(argumentName);
			argumentPair.setValue(argumentValue);
			argumentPairMap.put(argumentName, argumentPair);
		}
	}

	public static ArgumentPairPattern parse(String[] arguments) {
		ArgumentPairPattern instance = new ArgumentPairPattern();
		instance.initialize(arguments);
		return instance;
	}

	public String getArgumentValue(String argumentName) {
		ArgumentPair argumentPair = argumentPairMap.get(argumentName);
		return argumentPair == null ? null : argumentPair.getValue();
	}

	public Iterator<ArgumentPair> iterator() {
		final Iterator<Entry<String, ArgumentPair>> argumentPairEntryIterator = argumentPairMap.entrySet().iterator();
		return new Iterator<ArgumentPair>() {

			public boolean hasNext() {
				return argumentPairEntryIterator.hasNext();
			}

			public ArgumentPair next() {
				Entry<String, ArgumentPair> argumentPairEntry = argumentPairEntryIterator.next();
				ArgumentPair argumentPair = argumentPairEntry.getValue();
				return argumentPair;
			}

			public void remove() {
				argumentPairEntryIterator.remove();
			}
		};
	}

	public static void main(String[] args) {
		try {
			ArgumentPairPattern instance = ArgumentPairPattern.parse(args);
			for (ArgumentPair argumentPair : instance) {
				System.out.println(argumentPair);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
}
