package com.llw.libjava.io.compress;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.zip.GZIPInputStream;
import java.util.zip.GZIPOutputStream;

/**
 * @Author ZhuBolin
 * @Description gzip压缩工具类
 */
public class GzipUtil {
	private int bufferSize = 32768;// default,32KB

	public GzipUtil() {
	}

	public GzipUtil(int bufferSize) {
		setBufferSize(bufferSize);
	}

	public int getBufferSize() {
		return bufferSize;
	}

	public void setBufferSize(int bufferSize) {
		this.bufferSize = bufferSize;
	}

	public byte[] gzip(byte[] byteArray) throws IOException {
		return execGzip(byteArray);
	}

	public byte[] gunzip(byte[] byteArrayCompressed) throws IOException {
		return execGunzip(byteArrayCompressed, bufferSize);
	}

	/**
	 * @Author ZhuBolin
	 * @Description gzip压缩
	 * @param byteArray
	 *            待压缩的数据（字节数组形式）
	 * @return 压缩后的数据（字节数组形式）
	 * @throws IOException
	 */
	public static byte[] execGzip(byte[] byteArray) throws IOException {
		GZIPOutputStream gzipOutputStream = null;
		try {
			ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream(byteArray.length);
			gzipOutputStream = new GZIPOutputStream(byteArrayOutputStream);
			gzipOutputStream.write(byteArray);
			gzipOutputStream.finish();
			gzipOutputStream.flush();

			byte[] byteArrayCompressed = byteArrayOutputStream.toByteArray();
			return byteArrayCompressed;
		} finally {
			if (gzipOutputStream != null) {
				gzipOutputStream.close();
			}
		}
	}

	/**
	 * @Author ZhuBolin
	 * @Description gunzip解压缩
	 * @param byteArrayCompressed
	 *            被压缩的数据（字节数组形式）
	 * @param bufferSize
	 *            解压缩过程的buffer大小
	 * @return 还原的数据（字节数组形式）
	 * @throws IOException
	 */
	public static byte[] execGunzip(byte[] byteArrayCompressed, int bufferSize) throws IOException {
		GZIPInputStream gzipInputStream = null;
		try {
			ByteArrayInputStream byteArrayInputStream = new ByteArrayInputStream(byteArrayCompressed);
			gzipInputStream = new GZIPInputStream(byteArrayInputStream);
			byte[] buffer = new byte[bufferSize];
			int readInSize = 0;
			ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream(byteArrayCompressed.length);
			while ((readInSize = gzipInputStream.read(buffer)) > 0) {
				byteArrayOutputStream.write(buffer, 0, readInSize);
			}
			byte[] byteArrayUncompressed = byteArrayOutputStream.toByteArray();
			return byteArrayUncompressed;
		} finally {
			if (gzipInputStream != null) {
				gzipInputStream.close();
			}
		}
	}
}
