package com.llw.libjava.io.file;

import java.io.File;
import java.io.FileFilter;
import java.util.ArrayList;
import java.util.List;

import com.llw.libjava.commons.constant.SymbolConstants;
import com.llw.libjava.commons.util.ExceptionUtil;

public class FileUtil {

	private FileUtil() {
	}

	public static void assertFile(File file) {
		if (!file.isFile()) {
			ExceptionUtil.throwIllegalArgumentException("The file is not a file!");
		}
	}

	public static void assertDirectory(File directory) {
		if (!directory.isDirectory()) {
			ExceptionUtil.throwIllegalArgumentException("The directory is not a directory!");
		}
	}

	public static String getFileNameSuffix(File file) {
		if (file.exists()) {
			assertFile(file);
		}
		String fileName = file.getName();
		int dotIndex = fileName.lastIndexOf(SymbolConstants.PUNCTUATION_DOT);
		if (dotIndex > -1) {
			int suffixIndex = dotIndex + 1;
			if (suffixIndex < fileName.length()) {
				return fileName.substring(suffixIndex).toLowerCase();
			}
		}
		return null;
	}
	
	public static List<File> findByNameFilter(File directory, FileFilter fileFilter) {
		assertDirectory(directory);
		File[] subFiles = directory.listFiles();
		if (subFiles == null || subFiles.length == 0) {
			return null;
		}
		List<File> fileList = new ArrayList<File>();
		for (File subFile : subFiles) {
			if (subFile.isDirectory()) {
				List<File> subFileList = findByNameFilter(subFile, fileFilter);
				if (subFileList != null) {
					fileList.addAll(subFileList);
				}
			} else if (fileFilter.accept(subFile)) {
				fileList.add(subFile);
			}
		}
		return fileList;
	}

	public static void main(String[] args) {
		System.out.println(getFileNameSuffix(new File("a.")));
		System.out.println(getFileNameSuffix(new File("a.b")));
	}
}
