package com.llw.libjava.io.stream;

import java.io.IOException;
import java.io.InputStream;

public class ISLinkedInputStream extends InputStream {
	private InputStream baseInputStream = null;
	private InputStream appendInputStream = null;

	ISLinkedInputStream(InputStream baseInputStream, InputStream appendInputStream) {
		if (baseInputStream == null || appendInputStream == null) {
			throw new IllegalArgumentException("null");
		}
		this.baseInputStream = baseInputStream;
		this.appendInputStream = appendInputStream;
	}

	@Override
	public int read() throws IOException {
		int readIn = baseInputStream.read();
		if (readIn != -1) {
			return readIn;
		}
		return appendInputStream.read();
	}

	@Override
	public int available() throws IOException {
		return baseInputStream.available() + appendInputStream.available();
	}

	@Override
	public boolean markSupported() {
		return baseInputStream.markSupported() && appendInputStream.markSupported();
	}

	@Override
	public synchronized void reset() throws IOException {
		baseInputStream.reset();
		appendInputStream.reset();
	}

	@Override
	public void close() throws IOException {
		try {
			baseInputStream.close();
		} finally {
			appendInputStream.close();
		}
	}
}
