package security.cipher4j.base.util;

import java.security.Key;
import java.util.ArrayList;
import java.util.List;

import javax.crypto.Cipher;

import security.cipher4j.base.attr.CipherComputeAttribute;

public class CipherUtil {

	private CipherUtil() {
	}

	public static byte[] encryptDirectly(String algorithmName, Key key, byte[] cleartextBytes) throws Exception {
		Cipher cipher = Cipher.getInstance(algorithmName);
		cipher.init(Cipher.ENCRYPT_MODE, key);
		byte[] encryptedBytes = encryptDirectly(cipher, cleartextBytes);
		return encryptedBytes;
	}

	public static byte[] encryptDirectly(Cipher cipherEncrypt, byte[] cleartextBytes) throws Exception {
		byte[] encryptedBytes = cipherEncrypt.doFinal(cleartextBytes);
		return encryptedBytes;
	}

	public static byte[] encrypt(CipherComputeAttribute attribute, Key key, byte[] cleartextBytes) throws Exception {
		return encrypt(attribute.getAlgorithmName(), attribute.getCleartextBufferSize(), attribute.getCiphertextBufferSize(), key, cleartextBytes);
	}

	public static byte[] encrypt(String algorithmName, int cleartextBufferSize, int ciphertextBufferSize, Key key, byte[] cleartextBytes) throws Exception {
		Cipher cipherEncrypt = Cipher.getInstance(algorithmName);
		cipherEncrypt.init(Cipher.ENCRYPT_MODE, key);
		return encrypt(cipherEncrypt, cleartextBufferSize, ciphertextBufferSize, cleartextBytes);
	}

	public static byte[] encrypt(Cipher cipherEncrypt, int cleartextBufferSize, int ciphertextBufferSize, byte[] cleartextBytes) throws Exception {
		byte[] bytesEncrypted = transformCipherBuffer(cipherEncrypt, cleartextBufferSize, ciphertextBufferSize, cleartextBytes);
		return bytesEncrypted;
	}

	public static byte[] decryptDirectly(String algorithmName, Key key, byte[] cleartextBytes) throws Exception {
		Cipher cipher = Cipher.getInstance(algorithmName);
		cipher.init(Cipher.DECRYPT_MODE, key);
		byte[] decryptedBytes = cipher.doFinal(cleartextBytes);
		return decryptedBytes;
	}

	public static byte[] decryptDirectly(Cipher cipherDecrypt, byte[] cleartextBytes) throws Exception {
		byte[] decryptedBytes = cipherDecrypt.doFinal(cleartextBytes);
		return decryptedBytes;
	}

	public static byte[] decrypt(CipherComputeAttribute attribute, Key key, byte[] ciphertextBytes) throws Exception {
		return decrypt(attribute.getAlgorithmName(), attribute.getCleartextBufferSize(), attribute.getCiphertextBufferSize(), key, ciphertextBytes);
	}

	public static byte[] decrypt(String algorithmName, int cleartextBufferSize, int ciphertextBufferSize, Key key, byte[] ciphertextBytes) throws Exception {
		Cipher cipherDecrypt = Cipher.getInstance(algorithmName);
		cipherDecrypt.init(Cipher.DECRYPT_MODE, key);
		return decrypt(cipherDecrypt, cleartextBufferSize, ciphertextBufferSize, ciphertextBytes);
	}

	public static byte[] decrypt(Cipher cipherDecrypt, int cleartextBufferSize, int ciphertextBufferSize, byte[] ciphertextBytes) throws Exception {
		byte[] bytesDecrypted = transformCipherBuffer(cipherDecrypt, ciphertextBufferSize, cleartextBufferSize, ciphertextBytes);
		return bytesDecrypted;
	}

	private static byte[] transformCipherBuffer(Cipher cipher, int sourceBufferSize, int targetBufferSize, byte[] sourceBytes) throws Exception {
		int sourceBufferCount = sourceBytes.length / sourceBufferSize;
		int sourceBufferLeftLength = sourceBytes.length % sourceBufferSize;

		if (sourceBufferCount == 0) {
			byte[] sourceBuffer = new byte[sourceBufferLeftLength];
			int sourceOffset = sourceBufferCount * sourceBufferSize;
			System.arraycopy(sourceBytes, sourceOffset, sourceBuffer, 0, sourceBufferLeftLength);
			byte[] targetBuffer = cipher.doFinal(sourceBuffer);
			return targetBuffer;
		} else {
			List<Byte> targetByteList = new ArrayList<Byte>();
			for (int i = 0; i < sourceBufferCount; i++) {
				byte[] sourceBuffer = new byte[sourceBufferSize];
				int sourceOffset = i * sourceBufferSize;
				System.arraycopy(sourceBytes, sourceOffset, sourceBuffer, 0, sourceBufferSize);
				byte[] targetBuffer = cipher.doFinal(sourceBuffer);
				int targetBufferLength = targetBuffer.length;
				for (int j = 0; j < targetBufferLength; j++) {
					targetByteList.add(targetBuffer[j]);
				}
			}
			if (sourceBufferLeftLength > 0) {
				byte[] sourceBuffer = new byte[sourceBufferLeftLength];
				int sourceOffset = sourceBufferCount * sourceBufferSize;
				System.arraycopy(sourceBytes, sourceOffset, sourceBuffer, 0, sourceBufferLeftLength);
				byte[] targetBuffer = cipher.doFinal(sourceBuffer);
				int targetBufferLength = targetBuffer.length;
				for (int j = 0; j < targetBufferLength; j++) {
					targetByteList.add(targetBuffer[j]);
				}
			}
			int targetBytesLength = targetByteList.size();
			byte[] targetBytes = new byte[targetBytesLength];
			for (int i = 0; i < targetBytesLength; i++) {
				targetBytes[i] = targetByteList.get(i);
			}
			return targetBytes;
		}
	}

	@Deprecated
	public static byte[] transformCipherBuffer(Cipher cipher, Key key, byte[] sourceBytes, int sourceBufferSize, int targetBufferSize) throws Exception {
		return transformCipherBuffer(cipher, sourceBufferSize, targetBufferSize, sourceBytes);
	}
}
